
package SQL::Admin::Driver::DB2::Decompose;
use base qw( SQL::Admin::Driver::Base::Decompose );

use strict;
use warnings;

our $VERSION = v0.5.0;

######################################################################

package SQL::Admin::Driver::DB2::Decompose;

1;
